searchData={"items":[{"type":"module","title":"Turnstile","doc":"Use Cloudflare Turnstile in Phoenix. Check out the [README](readme.html) to get started.","ref":"Turnstile.html"},{"type":"function","title":"Turnstile.refresh/2","doc":"Refreshes the Turnstile widget in a LiveView.\n\nSince the widget uses `phx-update=\"ignore\"`, this function can be used if the widget needs to be\nre-mounted in the DOM, such as when the verification fails. If there are multiple Turnstile\nwidgets on the page and you only want to refresh one of them, pass a DOM ID as the second\nargument. Otherwise they will all be refreshed.","ref":"Turnstile.html#refresh/2"},{"type":"function","title":"Turnstile.remove/2","doc":"Removes the Turnstile widget from a LiveView.\n\nSince the widget uses `phx-update=\"ignore\"`, this function can be used if the widget needs to be\nremoved from the DOM. If there are multiple Turnstile widgets on the page and you only want to\nrefresh one of them, pass a DOM ID as the second argument. Otherwise they will all be removed.","ref":"Turnstile.html#remove/2"},{"type":"function","title":"Turnstile.script/1","doc":"Renders the Turnstile script tag.\n\nUses explicit rendering so it works with hooks. Additional attributes will be passed through to\nthe script tag.","ref":"Turnstile.html#script/1"},{"type":"function","title":"Turnstile.secret_key/0","doc":"Returns the configured secret key.","ref":"Turnstile.html#secret_key/0"},{"type":"function","title":"Turnstile.site_key/0","doc":"Returns the configured site key.","ref":"Turnstile.html#site_key/0"},{"type":"function","title":"Turnstile.verify/2","doc":"Calls the Turnstile verify endpoint with a response token.\n\nExpects a map with string keys that contains a value for `\"cf-response-token\"` (see\n[verification](readme.html#verification) for more info). The remote IP can be passed for extra\nsecurity when running the verification, but is optional. Returns `{:ok, response}` if the\nverification succeeded, or `{:error, reason}` if the verification failed.","ref":"Turnstile.html#verify/2"},{"type":"function","title":"Turnstile.widget/1","doc":"Renders the Turnstile widget.","ref":"Turnstile.html#widget/1"},{"type":"function","title":"Attributes - Turnstile.widget/1","doc":"* `:id` - The ID of the element. Defaults to `\"cf-turnstile\"`.\n  * `:class` - The class name passed to the element. Defaults to `nil`.\n  * `:hook` - The phx-hook used. Defaults to `\"Turnstile\"`.\n  * `:sitekey` - The Turnstile site key. Defaults to the `:site_key` config value.\n  * `:events` - An atom list of the callback events to listen for for in the Live View. See [events](readme.html#events).\n\nAll other attributes will be passed through to the element as `data-*` attributes so the widget\ncan be customized. See the [Turnstile docs](https://developers.cloudflare.com/turnstile/get-started/client-side-rendering/#configurations)\nfor a list of available attributes.","ref":"Turnstile.html#widget/1-attributes"},{"type":"behaviour","title":"Turnstile.Behaviour","doc":"","ref":"Turnstile.Behaviour.html"},{"type":"callback","title":"Turnstile.Behaviour.refresh/1","doc":"","ref":"Turnstile.Behaviour.html#c:refresh/1"},{"type":"callback","title":"Turnstile.Behaviour.refresh/2","doc":"","ref":"Turnstile.Behaviour.html#c:refresh/2"},{"type":"callback","title":"Turnstile.Behaviour.remove/1","doc":"","ref":"Turnstile.Behaviour.html#c:remove/1"},{"type":"callback","title":"Turnstile.Behaviour.remove/2","doc":"","ref":"Turnstile.Behaviour.html#c:remove/2"},{"type":"callback","title":"Turnstile.Behaviour.secret_key/0","doc":"","ref":"Turnstile.Behaviour.html#c:secret_key/0"},{"type":"callback","title":"Turnstile.Behaviour.site_key/0","doc":"","ref":"Turnstile.Behaviour.html#c:site_key/0"},{"type":"callback","title":"Turnstile.Behaviour.verify/1","doc":"","ref":"Turnstile.Behaviour.html#c:verify/1"},{"type":"callback","title":"Turnstile.Behaviour.verify/2","doc":"","ref":"Turnstile.Behaviour.html#c:verify/2"},{"type":"extras","title":"Phoenix Turnstile","doc":"# Phoenix Turnstile\n\nPhoenix components and helpers for using CAPTCHAs with [Cloudflare Turnstile](https://www.cloudflare.com/products/turnstile/). Before getting started, log into the Cloudflare dashboard and visit the Turnstile tab. Add a new site with your domain name (no need to add `localhost` if using the default test keys), and take note of your site key and secret key. You'll need these values later.\n\n- [Installation](#installation)\n- [Getting Started](#getting-started)\n- [Verification](#verification)\n- [Events](#events)\n- [Without LiveView](#without-liveview)\n- [Content Security Policies](#content-security-policies)\n- [Writing Tests](#writing-tests)","ref":"readme.html"},{"type":"extras","title":"Installation - Phoenix Turnstile","doc":"```elixir\ndef deps do\n  [\n    {:phoenix_turnstile, \"~> 1.0\"}\n  ]\nend\n```\n\nNow add the site key and secret key to your environment variables, and configure them in `config/runtime.exs`:\n\n```elixir\nconfig :phoenix_turnstile,\n  site_key: System.fetch_env!(\"TURNSTILE_SITE_KEY\"),\n  secret_key: System.fetch_env!(\"TURNSTILE_SECRET_KEY\")\n```\n\nYou don't need to add a site key or secret key for dev/test environments. This library will use the Turnstile test keys by default.","ref":"readme.html#installation"},{"type":"extras","title":"Getting Started - Phoenix Turnstile","doc":"To use CAPTCHAs in a LiveView app, start out by adding the script component in your root layout:\n\n```heex\n \n  <!-- ... -->\n\n   \n \n```\n\nNext, install the hook in `app.js` or wherever your live socket is being defined (make sure you're setting `NODE_PATH` in your [esbuild config](https://github.com/phoenixframework/esbuild#adding-to-phoenix) and including the `deps` folder):\n\n```javascript\nimport { TurnstileHook } from \"phoenix_turnstile\"\n\nconst liveSocket = new LiveSocket(\"/live\", Socket, {\n  /* ... */\n  hooks: {\n    Turnstile: TurnstileHook\n  }\n})\n```\n\nNow you can use the Turnstile widget component in any of your forms. For example:\n\n```heex\n<.form for={@form} phx-submit=\"submit\">\n   \n\n   Submit \n</.form>\n```\n\nTo customize the widget, pass any of the render parameters [specificed here](https://developers.cloudflare.com/turnstile/get-started/client-side-rendering/#configurations) (without the `data-` prefix).","ref":"readme.html#getting-started"},{"type":"extras","title":"Multiple Widgets - Phoenix Turnstile","doc":"If you want to have multiple widgets on the same page, pass a unique ID to `Turnstile.widget/1`, `Turnstile.refresh/1`, and `Turnstile.remove/1`.","ref":"readme.html#multiple-widgets"},{"type":"extras","title":"Verification - Phoenix Turnstile","doc":"The widget by itself won't actually complete the verification. It works by generating a token which gets injected into your form as a hidden input named `cf-turnstile-response`. The token needs to be sent to the Cloudflare API for final verification before continuing with the form submission. This should be done in your submit event using `Turnstile.verify/2`:\n\n```elixir\ndef handle_event(\"submit\", values, socket) do\n  case Turnstile.verify(values) do\n    {:ok, _} ->\n      # Verification passed!\n\n      {:noreply, socket}\n\n    {:error, _} ->\n      socket =\n        socket\n        |> put_flash(:error, \"Please try submitting again\")\n        |> Turnstile.refresh()\n\n      {:noreply, socket}\n  end\nend\n```\n\nTo be extra sure the user is not a robot, you also have the option of passing their IP address to the verification API. **This step is optional.** To get the user's IP address in LiveView, add `:peer_data` to the connect info for your socket in `endpoint.ex`:\n\n```elixir\nsocket \"/live\", Phoenix.LiveView.Socket,\n  websocket: [\n    connect_info: [:peer_data, ...]\n  ]\n```\n\nand pass it as the second argument to `Turnstile.verify/2`:\n\n```elixir\ndef mount(_params, session, socket) do\n  remote_ip = get_connect_info(socket, :peer_data).address\n  {:ok, assign(socket, :remote_ip, remote_ip)}\nend\n\ndef handle_event(\"submit\", values, socket) do\n  case Turnstile.verify(values, socket.assigns.remote_ip) do\n    # ...\n  end\nend\n```","ref":"readme.html#verification"},{"type":"extras","title":"Events - Phoenix Turnstile","doc":"The Turnstile widget supports the following events:\n\n* `:success` - When the challenge was successfully completed\n* `:error` - When there was an error (like a network error or the challenge failed)\n* `:expired` - When the challenge token expires and was not automatically reset\n* `:beforeInteractive` - Before the challenge enters interactive mode\n* `:afterInteractive` - After the challenge has left interactive mode\n* `:unsupported` - When a given client/browser is not supported by Turnstile\n* `:timeout` - When the challenge expires (after 5 minutes)\n\nThese can be useful for doing things like disabling the submit button until the challenge successfully completes, or refreshing the widget if it fails. To handle an event, add it to the `events` attribute and create a Turnstile event handler in the LiveView:\n\n```heex\n \n```\n\n```elixir\nhandle_event(\"turnstile:success\", _params, socket) do\n  # ...\n\n  {:noreply, socket}\nend\n```","ref":"readme.html#events"},{"type":"extras","title":"Without LiveView - Phoenix Turnstile","doc":"`Turnstile.script/1` and `Turnstile.widget/1` both rely on [client hooks](https://hexdocs.pm/phoenix_live_view/js-interop.html#client-hooks-via-phx-hook), and should work in non-LiveView pages as long as `app.js` is opening a live socket (which it should by default). Simply call `Turnstile.verify/2` in the controller:\n\n```elixir\ndef create(conn, params) do\n  case Turnstile.verify(params, conn.remote_ip) do\n    {:ok, _} ->\n      # Verification passed!\n\n      redirect(conn, to: ~p\"/success\")\n\n    {:error, _} ->\n      conn\n      |> put_flash(:error, \"Please try submitting again\")\n      |> redirect(to: ~p\"/new\")\n  end\nend\n```\n\nIf your page doesn't open a live socket or your're not using HEEx, you can still run Turnstile verifications by building your own client-side widget following the [documentation](https://developers.cloudflare.com/turnstile/get-started/client-side-rendering/) and using `Turnstile.site_key/0` to get your site key in the template:\n\n```elixir\ndef new(conn, _params) do\n  conn\n  |> assign(:site_key, Turnstile.site_key())\n  |> render(\"new.html\")\nend\n```\n\n```html\n \n  <!-- ... -->\n\n    \n   Submit \n \n```","ref":"readme.html#without-liveview"},{"type":"extras","title":"Content Security Policies - Phoenix Turnstile","doc":"If your site uses a content security policy, you'll need to add `https://challenges.cloudflare.com` to your `script-src` and `frame-src` directives. You can also add attributes to the script component such as `nonce`, and they will be passed through to the script tag:\n\n```heex\n \n  <!-- ... -->\n\n   \n \n```","ref":"readme.html#content-security-policies"},{"type":"extras","title":"Writing Tests - Phoenix Turnstile","doc":"When testing forms that use Turnstile verification, you may or may not want to call the live API.\n\nAlthough we use the test keys by default, you should consider using mocks during testing. An excellent library to consider is [mox](https://github.com/dashbitco/mox). Phoenix Turnstile exposes [`Turnstile.Behavior`](Turnstile.Behaviour.html) which makes writing tests much easier.\n\nTo start using Mox with Phoenix Turnstile, add this to your `test/test_helper.ex`:\n\n```elixir\nMox.defmock(TurnstileMock, for: Turnstile.Behaviour)\n```\n\nThen in your `config/test.exs`:\n\n```elixir\nconfig :phoenix_turnstile, adapter: TurnstileMock\n```\n\nTo make sure you're using `TurnstileMock` during testing, use the adapter from the config rather than using `Turnstile` directly:\n\n```elixir\n@turnstile Application.compile_env(:phoenix_turnstile, :adapter, Turnstile)\n\ndef handle_event(\"submit\", values, socket) do\n  case @turnstile.verify(values) do\n    {:ok, _} ->\n      # Verification passed!\n\n      {:noreply, socket}\n\n    {:error, _} ->\n      socket =\n        socket\n        |> put_flash(:error, \"Please try submitting again\")\n        |> @turnstile.refresh()\n\n      {:noreply, socket}\n  end\nend\n```\n\nNow you can easily mock or stub any Turnstile function in your tests and they won't make any real API calls:\n\n```elixir\nimport Mox\n\nsetup do\n  stub(TurnstileMock, :refresh, fn socket -> socket end)\n  stub(TurnstileMock, :verify, fn _values, _remoteip -> {:ok, %{}} end)\nend\n```","ref":"readme.html#writing-tests"}],"content_type":"text/markdown","proglang":"elixir","producer":{"name":"ex_doc","version":"0.37.0"}}